#version 330 core

uniform sampler2D textureID;

uniform float xOffset;
uniform float yOffset;
const  float fxaaQualitySubpix = 0.75;
const float fxaaQualityEdgeThreshold = 0.166;
const float fxaaQualityEdgeThresholdMin = 0.0833;

in vec2 posM;

out vec4 out0;

const  float FXAA_QUALITY_PS = 6;
const  float FXAA_QUALITY_PRESET = 23;
const  float FXAA_QUALITY_P0 = 1.0;
const  float FXAA_QUALITY_P1 = 1.5;
const  float FXAA_QUALITY_P2 = 2.0;
const  float FXAA_QUALITY_P3 = 2.0;
const  float FXAA_QUALITY_P4 = 2.0;
const  float FXAA_QUALITY_P5 = 8.0;
const  float FXAA_QUALITY_P6 = 0.0;
const  float FXAA_QUALITY_P7 = 0.0;
const  float FXAA_QUALITY_P8 = 0.0;
const  float FXAA_QUALITY_P9 = 0.0;
const  float FXAA_QUALITY_P10 = 0.0;
const  float FXAA_QUALITY_P11 = 0.0;
const  float FXAA_QUALITY_P12 = 0.0;

void main(void)
{
			vec4 rgbyM = textureLod(textureID, posM, 0.0);

			float lumaM = rgbyM.a;

			float lumaS = textureLodOffset(textureID, posM, 0.0, ivec2( 0, 1)).a;
			float lumaE = textureLodOffset(textureID, posM, 0.0, ivec2( 1, 0)).a;
			float lumaN = textureLodOffset(textureID, posM, 0.0, ivec2( 0,-1)).a;
			float lumaW = textureLodOffset(textureID, posM, 0.0, ivec2(-1, 0)).a;
	/*--------------------------------------------------------------------------*/
		float maxSM = max(lumaS, lumaM);
		float minSM = min(lumaS, lumaM);
		float maxESM = max(lumaE, maxSM);
		float minESM = min(lumaE, minSM);
		float maxWN = max(lumaN, lumaW);
		float minWN = min(lumaN, lumaW);
		float rangeMax = max(maxWN, maxESM);
		float rangeMin = min(minWN, minESM);
		float rangeMaxScaled = rangeMax * fxaaQualityEdgeThreshold;
		float range = rangeMax - rangeMin;
		float rangeMaxClamped = max(fxaaQualityEdgeThresholdMin, rangeMaxScaled);
		bool earlyExit = range < rangeMaxClamped;
	/*--------------------------------------------------------------------------*/
		
		if(earlyExit)
		{
			out0 = 1.055 * pow(rgbyM,  vec4(0.41666)) - 0.055;
		}
		else
		{
		
	/*--------------------------------------------------------------------------*/
			float lumaNW = textureLodOffset(textureID, posM, 0.0, ivec2(-1,-1)).a;
			float lumaSE = textureLodOffset(textureID, posM, 0.0, ivec2( 1, 1)).a;
			float lumaNE = textureLodOffset(textureID, posM, 0.0, ivec2( 1,-1)).a;
			float lumaSW = textureLodOffset(textureID, posM, 0.0, ivec2(-1, 1)).a;

	/*--------------------------------------------------------------------------*/
		float lumaNS = lumaN + lumaS;
		float lumaWE = lumaW + lumaE;
		float subpixRcpRange = 1.0/range;
		float subpixNSWE = lumaNS + lumaWE;
		float edgeHorz1 = (-2.0 * lumaM) + lumaNS;
		float edgeVert1 = (-2.0 * lumaM) + lumaWE;
	/*--------------------------------------------------------------------------*/
		float lumaNESE = lumaNE + lumaSE;
		float lumaNWNE = lumaNW + lumaNE;
		float edgeHorz2 = (-2.0 * lumaE) + lumaNESE;
		float edgeVert2 = (-2.0 * lumaN) + lumaNWNE;
	/*--------------------------------------------------------------------------*/
		float lumaNWSW = lumaNW + lumaSW;
		float lumaSWSE = lumaSW + lumaSE;
		float edgeHorz4 = (abs(edgeHorz1) * 2.0) + abs(edgeHorz2);
		float edgeVert4 = (abs(edgeVert1) * 2.0) + abs(edgeVert2);
		float edgeHorz3 = (-2.0 * lumaW) + lumaNWSW;
		float edgeVert3 = (-2.0 * lumaS) + lumaSWSE;
		float edgeHorz = abs(edgeHorz3) + edgeHorz4;
		float edgeVert = abs(edgeVert3) + edgeVert4;
	/*--------------------------------------------------------------------------*/
		float subpixNWSWNESE = lumaNWSW + lumaNESE;
		float lengthSign = xOffset;
		bool horzSpan = edgeHorz >= edgeVert;
		float subpixA = subpixNSWE * 2.0 + subpixNWSWNESE;
	/*--------------------------------------------------------------------------*/
		if(!horzSpan) lumaN = lumaW;
		if(!horzSpan) lumaS = lumaE;
		if(horzSpan) lengthSign = yOffset;
		float subpixB = (subpixA * (1.0/12.0)) - lumaM;
	/*--------------------------------------------------------------------------*/
		float gradientN = lumaN - lumaM;
		float gradientS = lumaS - lumaM;
		float lumaNN = lumaN + lumaM;
		float lumaSS = lumaS + lumaM;
		bool pairN = abs(gradientN) >= abs(gradientS);
		float gradient = max(abs(gradientN), abs(gradientS));
		if(pairN) lengthSign = -lengthSign;
		float subpixC = clamp((abs(subpixB) * subpixRcpRange), 0.0, 1.0);
	/*--------------------------------------------------------------------------*/
		vec2 posB;
		posB.x = posM.x;
		posB.y = posM.y;
		vec2 offNP;
		offNP.x = (!horzSpan) ? 0.0 : xOffset;
		offNP.y = ( horzSpan) ? 0.0 : yOffset;
		if(!horzSpan) posB.x += lengthSign * 0.5;
		if( horzSpan) posB.y += lengthSign * 0.5;
	/*--------------------------------------------------------------------------*/
		vec2 posN;
		posN.x = posB.x - offNP.x * FXAA_QUALITY_P0;
		posN.y = posB.y - offNP.y * FXAA_QUALITY_P0;
		vec2 posP;
		posP.x = posB.x + offNP.x * FXAA_QUALITY_P0;
		posP.y = posB.y + offNP.y * FXAA_QUALITY_P0;
		float subpixD = ((-2.0)*subpixC) + 3.0;
		float lumaEndN = textureLod(textureID, posN, 0.0).a;
		float subpixE = subpixC * subpixC;
		float lumaEndP = textureLod(textureID, posP, 0.0).a;
	/*--------------------------------------------------------------------------*/
		if(!pairN) lumaNN = lumaSS;
		float gradientScaled = gradient * 1.0/4.0;
		float lumaMM = lumaM - lumaNN * 0.5;
		float subpixF = subpixD * subpixE;
		bool lumaMLTZero = lumaMM < 0.0;
	/*--------------------------------------------------------------------------*/
		lumaEndN -= lumaNN * 0.5;
		lumaEndP -= lumaNN * 0.5;
		bool doneN = abs(lumaEndN) >= gradientScaled;
		bool doneP = abs(lumaEndP) >= gradientScaled;
		if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P1;
		if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P1;
		bool doneNP = (!doneN) || (!doneP);
		if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P1;
		if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P1;
	/*--------------------------------------------------------------------------*/
		if(doneNP) {
			if(!doneN) lumaEndN = textureLod(textureID, posN.xy, 0.0).a;
			if(!doneP) lumaEndP = textureLod(textureID, posP.xy, 0.0).a;
			if(!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
			if(!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
			doneN = abs(lumaEndN) >= gradientScaled;
			doneP = abs(lumaEndP) >= gradientScaled;
			if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P2;
			if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P2;
			doneNP = (!doneN) || (!doneP);
			if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P2;
			if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P2;
	/*--------------------------------------------------------------------------*/
				if(FXAA_QUALITY_PS > 3 &&doneNP)
				{
					if(!doneN) lumaEndN = textureLod(textureID, posN.xy, 0.0).a;
					if(!doneP) lumaEndP = textureLod(textureID, posP.xy, 0.0).a;
					if(!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
					if(!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
					doneN = abs(lumaEndN) >= gradientScaled;
					doneP = abs(lumaEndP) >= gradientScaled;
					if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P3;
					if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P3;
					doneNP = (!doneN) || (!doneP);
					if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P3;
					if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P3;
			
	/*--------------------------------------------------------------------------*/
				if(FXAA_QUALITY_PS > 4 && doneNP) {
					if(!doneN) lumaEndN = textureLod(textureID, posN.xy, 0.0).a;
					if(!doneP) lumaEndP = textureLod(textureID, posP.xy, 0.0).a;
					if(!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
					if(!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
					doneN = abs(lumaEndN) >= gradientScaled;
					doneP = abs(lumaEndP) >= gradientScaled;
					if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P4;
					if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P4;
					doneNP = (!doneN) || (!doneP);
					if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P4;
					if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P4;
	/*--------------------------------------------------------------------------*/
					if(FXAA_QUALITY_PS > 5 && doneNP) {
						if(!doneN) lumaEndN = textureLod(textureID, posN.xy, 0.0).a;
						if(!doneP) lumaEndP = textureLod(textureID, posP.xy, 0.0).a;
						if(!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
						if(!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
						doneN = abs(lumaEndN) >= gradientScaled;
						doneP = abs(lumaEndP) >= gradientScaled;
						if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P5;
						if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P5;
						doneNP = (!doneN) || (!doneP);
						if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P5;
						if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P5;
	/*--------------------------------------------------------------------------*/
						if(FXAA_QUALITY_PS > 6 && doneNP) {
							if(!doneN) lumaEndN = textureLod(textureID, posN.xy, 0.0).a;
							if(!doneP) lumaEndP = textureLod(textureID, posP.xy, 0.0).a;
							if(!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
							if(!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
							doneN = abs(lumaEndN) >= gradientScaled;
							doneP = abs(lumaEndP) >= gradientScaled;
							if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P6;
							if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P6;
							doneNP = (!doneN) || (!doneP);
							if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P6;
							if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P6;
	/*--------------------------------------------------------------------------*/
							if(FXAA_QUALITY_PS > 7 && doneNP) {
								if(!doneN) lumaEndN = textureLod(textureID, posN.xy, 0.0).a;
								if(!doneP) lumaEndP = textureLod(textureID, posP.xy, 0.0).a;
								if(!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
								if(!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
								doneN = abs(lumaEndN) >= gradientScaled;
								doneP = abs(lumaEndP) >= gradientScaled;
								if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P7;
								if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P7;
								doneNP = (!doneN) || (!doneP);
								if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P7;
								if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P7;
	/*--------------------------------------------------------------------------*/
		if(FXAA_QUALITY_PS > 8 && doneNP) {
			if(!doneN) lumaEndN = textureLod(textureID, posN.xy, 0.0).a;
			if(!doneP) lumaEndP = textureLod(textureID, posP.xy, 0.0).a;
			if(!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
			if(!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
			doneN = abs(lumaEndN) >= gradientScaled;
			doneP = abs(lumaEndP) >= gradientScaled;
			if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P8;
			if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P8;
			doneNP = (!doneN) || (!doneP);
			if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P8;
			if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P8;
	/*--------------------------------------------------------------------------*/
			if(FXAA_QUALITY_PS > 9 && doneNP) {
				if(!doneN) lumaEndN = textureLod(textureID, posN.xy, 0.0).a;
				if(!doneP) lumaEndP = textureLod(textureID, posP.xy, 0.0).a;
				if(!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
				if(!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
				doneN = abs(lumaEndN) >= gradientScaled;
				doneP = abs(lumaEndP) >= gradientScaled;
				if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P9;
				if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P9;
				doneNP = (!doneN) || (!doneP);
				if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P9;
				if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P9;
	/*--------------------------------------------------------------------------*/
				if(FXAA_QUALITY_PS > 10 && doneNP) {
					if(!doneN) lumaEndN = textureLod(textureID, posN.xy, 0.0).a;
					if(!doneP) lumaEndP = textureLod(textureID, posP.xy, 0.0).a;
					if(!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
					if(!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
					doneN = abs(lumaEndN) >= gradientScaled;
					doneP = abs(lumaEndP) >= gradientScaled;
					if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P10;
					if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P10;
					doneNP = (!doneN) || (!doneP);
					if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P10;
					if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P10;
	/*--------------------------------------------------------------------------*/
					if(FXAA_QUALITY_PS > 11 && doneNP) {
						if(!doneN) lumaEndN = textureLod(textureID, posN.xy, 0.0).a;
						if(!doneP) lumaEndP = textureLod(textureID, posP.xy, 0.0).a;
						if(!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
						if(!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
						doneN = abs(lumaEndN) >= gradientScaled;
						doneP = abs(lumaEndP) >= gradientScaled;
						if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P11;
						if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P11;
						doneNP = (!doneN) || (!doneP);
						if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P11;
						if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P11;
	/*--------------------------------------------------------------------------*/
						if(FXAA_QUALITY_PS > 12 && doneNP) {
							if(!doneN) lumaEndN = textureLod(textureID, posN.xy, 0.0).a;
							if(!doneP) lumaEndP = textureLod(textureID, posP.xy, 0.0).a;
							if(!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
							if(!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
							doneN = abs(lumaEndN) >= gradientScaled;
							doneP = abs(lumaEndP) >= gradientScaled;
							if(!doneN) posN.x -= offNP.x * FXAA_QUALITY_P12;
							if(!doneN) posN.y -= offNP.y * FXAA_QUALITY_P12;
							doneNP = (!doneN) || (!doneP);
							if(!doneP) posP.x += offNP.x * FXAA_QUALITY_P12;
							if(!doneP) posP.y += offNP.y * FXAA_QUALITY_P12;

	/*--------------------------------------------------------------------------*/
	}}}}}}}}}}}	
							
	/*--------------------------------------------------------------------------*/
		float dstN = posM.x - posN.x;
		float dstP = posP.x - posM.x;
		if(!horzSpan)
			dstN = posM.y - posN.y;
		if(!horzSpan)
			dstP = posP.y - posM.y;
	/*--------------------------------------------------------------------------*/
		bool goodSpanN = (lumaEndN < 0.0) != lumaMLTZero;
		float spanLength = (dstP + dstN);
		bool goodSpanP = (lumaEndP < 0.0) != lumaMLTZero;
		float spanLengthRcp = 1.0/spanLength;
	/*--------------------------------------------------------------------------*/
		bool directionN = dstN < dstP;
		float dst = min(dstN, dstP);
		bool goodSpan = directionN ? goodSpanN : goodSpanP;
		float subpixG = subpixF * subpixF;
		float pixelOffset = (dst * (-spanLengthRcp)) + 0.5;
		float subpixH = subpixG * fxaaQualitySubpix;
	/*--------------------------------------------------------------------------*/
		float pixelOffsetGood = goodSpan ? pixelOffset : 0.0;
		float pixelOffsetSubpix = max(pixelOffsetGood, subpixH);
		vec2 newPos = posM;
		if(!horzSpan)
			newPos.x += pixelOffsetSubpix * lengthSign;
		if( horzSpan)
			newPos.y += pixelOffsetSubpix * lengthSign;

		
		out0 =  1.055 * pow(texture(textureID, newPos),  vec4(0.41666)) - 0.055;
	   }
} 