#version 330 core

uniform sampler2D tex_color;
uniform sampler2D tex_normal;
uniform sampler2D tex_depth;
uniform int hasNormalMap;
uniform vec2 scaleBias;

uniform float specular;

in vec2 ex_texCoord;
in mat3 ex_TBNMatrix;
in vec3 ex_eyeVecTan;

out	vec4 out0; // Albedo + Specular
out	vec4 out1; // Normal + Depth
out vec4 out2; // Glow texture


vec3 PrepareEyeVec(vec3 eyeVec)
{
	vec3 preparedEyeVec = normalize(eyeVec);	
	preparedEyeVec.z = abs(preparedEyeVec.z);
	preparedEyeVec.xy *= 0.2;
	preparedEyeVec /= preparedEyeVec.z;
	return preparedEyeVec;
}


void main(void)
{
	vec2 newTexCoords = ex_texCoord;
	
	if(hasNormalMap >= 1)
	{
		if(hasNormalMap >= 3)
		{
			//Start Pyramidical Displacement Mapping / Quadtree Displacement Mapping
			vec3 eyeVec = PrepareEyeVec(ex_eyeVecTan);
			
			vec2 dirSign = sign(eyeVec.xy) * 0.5 + 0.5;
			const float texelSpanHalf = 0.5 / 1024.0;	
			float cellOffset = texelSpanHalf * 0.5;
			
			int i = 0;
			int level = 10;
				
			int cellCount = 1;
			vec3 startPoint = vec3(ex_texCoord, 0.0);
			float depth = texelFetch(tex_depth, ivec2(startPoint.xy * cellCount), 0).r;
			vec3 currentPoint = startPoint + eyeVec * depth;

			
			while(level > 0 && i < 25)
			{
				++i;
					
				--level;
				cellCount *= 2;
					
				depth = texelFetch(tex_depth, ivec2(currentPoint.xy * cellCount), level).r;
				
				if(currentPoint.z  < depth)
				{
					vec3 temporaryPoint = startPoint + eyeVec * depth;
					
					//Check if in same cell
					ivec4 cellResult = ivec4(currentPoint.xy * cellCount, temporaryPoint.xy * cellCount);

					if ( (cellResult.x == cellResult.z) && (cellResult.y == cellResult.w ) )
					{	//Both points are in the same cell
						currentPoint = temporaryPoint;
					}
					else
					{	//We cross the cell so we need to calculate the closest cell intersection so we won't skip height values
						
						//Calculate distance to nearest bound
						vec2 a = currentPoint.xy - startPoint.xy;
						vec2 p3 = (cellResult.xy + dirSign) / cellCount;
						vec2 b = p3.xy - startPoint.xy;

						vec2 dNC = (b.xy * currentPoint.z) / a.xy; 
						//take the nearest cell
						depth = min(depth,min(dNC.x, dNC.y))+cellOffset;
						currentPoint = startPoint + eyeVec * depth;
						
						++level;
						cellCount /= 2;
					}
				}
			}
			
			newTexCoords = currentPoint.xy;
						
	
			
	/*
			int cellCount = 1;
			float d = 0.0;
			vec3 p = vec3(ex_texCoord, 0.0);
			vec3 p2 = p;
			
			while (level >= 0.0 && 70 > i)
			{
				d = textureLod(tex_depth, p2.xy, level).r;
				
				if (d >= p2.z)
				{	
					//predictive point of ray traversal
					vec3 tmpP2 = p + eyeVec * d;

					//current and predictive node ID
					ivec4 nodeID = ivec4(p2.xy*cellCount, tmpP2.xy*cellCount);

					//check if we are crossing the current cell
					if (nodeID.x != nodeID.z || nodeID.y != nodeID.w)
					{    
						//calculate distance to nearest bound
						vec2 a = p2.xy - p.xy;
						vec2 p3 = (nodeID.xy + dirSign) / cellCount;
						vec2 b = p3.xy - p.xy;

						vec2 dNC = (b.xy * p2.z) / a.xy; 
						//take the nearest cell
						d = min(d,min(dNC.x, dNC.y))+cellOffset;
						
						level++;
						cellCount/=2;
					}
					p2 = p + eyeVec * d;
				}
				level--;
				cellCount*=2;
				i++;
			}
					
			newTexCoords = p2.xy;
			*/
		}
		
		
		// Calculating normals in view space for normals rendertarget
		vec4 normal = texture(tex_normal, newTexCoords);
		vec3 normalVec = normal.xyz * 2.0 - 1.0;
		out1.xyz = normalize(ex_TBNMatrix * normalVec);
		
		out0 = texture(tex_color, newTexCoords);

	}
	else
	{
		out1.xyz = ex_TBNMatrix[0];
		// Retrieving frag colour for writing into the albedo rendertarget
		out0 = texture(tex_color, newTexCoords);
	}
		

	
	//Glow texture
	out2 = vec4(out0.w * out0.rgb, 1.0);
}



