#version 330 core

uniform mat4 modelViewProjectionMatrix;
uniform mat3 modelViewInvTransposeMatrix3;
uniform int hasNormalMap;
uniform vec3 cameraPosition;

in vec3 in_position;
in vec3 in_normal;
in vec3 in_tangent;
in vec2 in_texcoord;

out vec2 ex_texCoord;
out mat3 ex_TBNMatrix;
out vec3 ex_eyeVecTan;

void main(void)
{
	ex_texCoord = in_texcoord;
	
	vec3 viewSpaceNormal = normalize(modelViewInvTransposeMatrix3 * in_normal);
	
	if(hasNormalMap == 1)
	{
		vec3 viewSpaceTangent = normalize(modelViewInvTransposeMatrix3 * in_tangent);
		vec3 viewSpaceBinormal = cross(viewSpaceNormal, viewSpaceTangent);
		
		ex_TBNMatrix = mat3(viewSpaceTangent, viewSpaceBinormal, viewSpaceNormal);
		
		ex_eyeVecTan = in_position.xyz - cameraPosition;
		
		mat3 tangentToWorld = mat3(in_tangent, cross(in_normal, in_tangent), in_normal);
		ex_eyeVecTan = ex_eyeVecTan * tangentToWorld;
	}
	else
	{
		ex_TBNMatrix[0] = viewSpaceNormal;
	}
	


	gl_Position = modelViewProjectionMatrix * vec4(in_position, 1.0);
}