#version 330 core

uniform sampler2D tex_albedo;
uniform sampler2D tex_normal;
uniform sampler2D tex_depth;
	
uniform vec3 view_lightPos;

uniform vec4 lightFalloff;
uniform vec2 spotLightParams;
	
uniform vec3 lightColour;

uniform float Amod;
uniform float Bmod;

// Attributes of light
in vec4 ex_viewPosition;
in vec4 ex_perspPosition;
	
out vec4 out0;
	
void main(void)
{
	vec2 texCoord = ex_perspPosition.xy / ex_perspPosition.w;
	vec4 normalsDepth = texture(tex_normal, texCoord); // Attribute 1: Normal+depth
	float depth = texture(tex_depth, texCoord).x;
	
	depth = Amod / (depth + Bmod);
	
	// Compute view position of fragment
    vec3 viewray = vec3( ex_viewPosition.xy  / ex_viewPosition.z , 1.0 );
    vec3 view_fragPosition = depth * viewray;

	// Calculate light direction and distance
	vec3 objToLightVec = view_lightPos - view_fragPosition;
	float len = length(objToLightVec);

	if(lightFalloff.x  >  len)
	{
		vec3 objToLightDir = objToLightVec / len;
		float lambertTerm = dot(objToLightDir, normalsDepth.xyz);
		
		if(lambertTerm > 0.0)
		{
			vec4 a0 = texture(tex_albedo, texCoord); // Attribute 0: Diffuse color+shininess
				
			
			//Diffuse
			vec3 finalLightColour = lambertTerm * lightColour;
			
			//Specular
			float p = 22.0;	
			vec3 h = normalize(normalize(viewray) + objToLightDir);
			finalLightColour += ((p + 2.0)/8.0) * pow(max(0.0, dot(normalsDepth.xyz, h)),p) * lightColour;
					
			float attenuation = dot(lightFalloff.yzw, vec3(1.0, len, len*len));
			out0 = vec4(finalLightColour * a0.rgb / attenuation, 1.0);
		}
		else
		{
			//Discard
			out0 = vec4(0.0);
		}
	}
	else
	{
		//Discard
		out0 = vec4(0.0);
	}
}
