#version 330 core

uniform sampler2D tex_albedo;
uniform sampler2D tex_normal;
uniform sampler2D tex_depth;
uniform sampler2D tex_lightMap;
uniform sampler2D tex_shadowMap;



uniform float lightBleedReduction;
uniform float minVariance;

uniform vec3 view_lightPos;
uniform vec3 view_lightDir;

uniform vec4 lightFalloff;
uniform vec2 spotLightParams;
	
uniform vec3 lightColour;

uniform int shadowMapping;
uniform mat4 eyeViewPositionToLightPerspPositionMatrix;
uniform mat4 projectionMatrix;

uniform float Amod;
uniform float Bmod;

// Attributes of light
in vec4 ex_viewPosition;
in vec4 ex_perspPosition;
	
out vec4 out0;





float ReduceLightBleeding(float p_max, float Amount)  
{  
     return smoothstep(Amount, 1.0, p_max);  
}  


float calculateShadow(vec4 shadowMapLookup, float distance)
{
	float m1 = shadowMapLookup.x;
	float m2 = shadowMapLookup.y;

	if( distance <= m1 )
		return 1.0;
	else
	{
		float var = m2 - (m1 * m1);
		var = max(var, minVariance);

		float d = distance - m1;
		float pMax = var / (var + d * d);
		pMax = ReduceLightBleeding(pMax, lightBleedReduction);
		return pMax;
	}
}

void main(void)
{
	vec2 texCoord =  ex_perspPosition.xy / ex_perspPosition.w;
	vec4 normalsDepth = texture(tex_normal, texCoord); // Attribute 1: Normal+depth
	float depth = texture(tex_depth, texCoord).x;
	depth = Amod / (depth + Bmod);
	
	// Compute view position of fragment
    vec3 viewray = vec3( ex_viewPosition.xy  / ex_viewPosition.z , 1.0 );
	
	vec3 view_fragPosition = depth * viewray;

	const float specularity = 0.83;
	
	// Calculate light direction and distance
	vec3 objToLightVec = view_lightPos - view_fragPosition;
	float len = length(objToLightVec);
	vec3 objToLightDir = objToLightVec / len;

	//Falloff & Attenuation pre-check to discard fragment
	float spotlightAngleDot = dot(view_lightDir, -objToLightDir);

	if(lightFalloff.x  >  len && spotlightAngleDot > spotLightParams.x)
	{
		float lambertTerm = dot(objToLightDir, normalsDepth.xyz);
		
		if(lambertTerm > 0.0)
		{
			
			vec4 lightViewProjPosition = eyeViewPositionToLightPerspPositionMatrix * vec4(view_fragPosition, 1.0);
	
			vec4 albedo = texture(tex_albedo, texCoord); // Attribute 0: Diffuse color+shininess
			float spotFalloff = textureProj(tex_lightMap, lightViewProjPosition).r;
				
			vec3 finalLightColour;
			//Diffuse
			finalLightColour = lambertTerm * lightColour;
			
			//Specular
			float specularity = 0.6;
			vec3 h = normalize(normalize(viewray) - objToLightDir);
			finalLightColour += specularity * pow(max(0.0, dot(normalsDepth.xyz, h)),22.0) * lightColour;
					

			float len_sq = len * len;
			//Distance attenuation + Spotlight falloff (angles)
			float lightStrength = spotFalloff / dot(lightFalloff.yzw, vec3(1.f, len, len_sq));
			
			float shadow = 1.0;
			
			if(shadowMapping == 1)
			{
				vec4 shadowMapLookup = textureProj(tex_shadowMap, lightViewProjPosition);
				
				// Compute the distance from the light to the current fragment.  
				float surfaceDistToLightNormalizedDistance = len;  
				
				shadow = calculateShadow(shadowMapLookup, surfaceDistToLightNormalizedDistance);
			}
			
			out0 = vec4( lightStrength * shadow * finalLightColour * albedo.rgb, 1.0);
		}
		else
		{
			//Discard
			out0 = vec4(0.0);
		}
	}
	else
	{
		//Discard
		out0 = vec4(0.0);
	}
}




